/*********************************************************************
 *                
 * Filename:      driver.h
 * Version:       
 * Description:   
 * Status:        Experimental.
 * Author:        Haris Zukanovic <haris@stud.cs.uit.no>
 * Created at:    Tue Apr 14 12:41:42 1998
 * Modified at:   Sat May 23 17:40:00 1998
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * Modified At:   Tue Aug 18 1998
 * Modified By:   Paul VanderSpek <pjjvande@undergrad.math.uwaterloo.ca>
 * 
 *     Copyright (c) 1998 Haris Zukanovic, All Rights Reserved.
 *      
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version.
 *  
 *     Neither Haris Zukanovic nor University of Troms admit liability nor
 *     provide warranty for any of this software. This material is 
 *     provided "AS-IS" and at no charge.
 *     
 ********************************************************************/

#ifndef DRIVER_H
#define DRIVER_H

#include <linux/timer.h>
#include <linux/tty.h>
#include <linux/fs.h>

#include "irda.h"

#define CHIP_CB_MAGIC	0x1961
#define BUFF_CB_MAGIC	0x1998
#define DRIVER_CB_MAGIC	0x3010

struct chipio_cb {
	int magic;

	int iobase;		/* IO base */  
	int ioextent;		/* # of io ports needed */
	int irq;		/* ACE irq */
	int fifo_size;		/* FIFO size */

	int dma;		/* DMA channel used */
	int irqflags;		/* interrupt flags (ie, SA_SHIRQ|SA_INTERRUPT) */
};

struct buff_cb {
	int magic;

	int state;
	int in_frame;

	__u8 *head;		/* where in the buffer we are */
	__u8 *buff;		/* the buffer */

	int ptr;		/* location of pointer into buffer */
	int count;		/* currently used bytes in buffer */
	int buffsize;		/* size of the buffer */
	__u16 fcs;

	/*  
	 *  And the list of outstanding skb's for device 
	 */
	struct sk_buff_head list;
};

#define XMIT	0x01
#define RCV	0x02

#define SIR_MODE 	0x01
#define MIR_MODE 	0x02
#define FIR_MODE 	0x04
#define IO_PIO   	0x08
#define IO_DMA		0x10

/* local flags */

#define ACE_IRQ_ALLOCATED	0x01
#define SCE_IRQ_ALLOCATED	0x02
#define DMA_ALLOCATED		0x04
#define ACE_IO_ALLOCATED	0x08
#define SCE_IO_ALLOCATED	0x10
#define HANDLE_IN_USE		0x20

#define IRVDD_REV	0x01
#define BUF_MAX_LEN	4096

struct driver_cb {
	int magic;	/* our magic bullet */
	int driver_id;	/* used to register the driver */
	int irvdd_rev;  /* virtual device layer rev */
	int revision;	/* what revision this driver is; we can force a certain rev */
	int handle;	/* what handle in the IrVDD structure */

	__u16 driver_flags;	/* driver capabilities/needs */
	__u16 flags;		/* IrVDD flags */

	char *driver_name;
	char *tty_name;
	struct chipio_cb ACE, SCE;
	struct buff_cb tx, rx;
	int media_busy;
	int baud;
	int direction;

	/* Media busy stuff */
	struct timer_list   media_busy_timer;
	struct timer_list   todo_timer;
	
	void *data;

	void (*hard_xmit)(struct driver_cb *driver,struct sk_buff *skb); /* send handler */
        void (*change_speed)(struct driver_cb *driver ,int baud); /* set speed */
	int (*is_media_busy)(struct driver_cb *driver);	   /* media busy? */
	void (*set_media_busy) (struct driver_cb *driver, int);
	int (*is_receiving)(struct driver_cb *driver);	   /* receiving? */
	int (*is_tbusy)(struct driver_cb *driver);	   /* transmitting? */

	void (*open) (struct driver_cb *driver, struct tty_struct *tty);
	void (*closed) (struct driver_cb *driver);

	void (*irq_handler) (int irq, void *dev_id, struct pt_regs *regs);
	void (*driver_bh) (void);

	/* DMA information */
	int (*dma_terminal_count) (void);
	int (*dma_setup_read) (int dma);
	int (*dma_setup_write) (int dma);
  
  /* Driver information */
  struct file_operations *fops;
  
  /* Used to block on reading */
  struct wait_queue *readq;
};

extern void FIR_new_frame(struct driver_cb *driver);
extern void FIR_eof(struct driver_cb *driver);
extern int FIR_xmit(struct driver_cb *driver);
//extern void FIR_receive(struct driver_cb *driver);
extern void FIR_setspeed(struct driver_cb *driver, int baud);
extern void FIR_shutdown(struct driver_cb *driver);
extern void FIR_interrupt(int irq, void *dev_id, struct pt_regs *regs);
extern int FIR_detect(struct driver_cb *driver);


#endif
