/*********************************************************************
 *                
 * Filename:      ircompat.h
 * Version:       
 * Description:   This file tries to help with the differences of the 
 *                various kernel versions.
 * Status:        Experimental.
 * Author:        Dag Brattli <dagb@cs.uit.no>
 * Created at:    Sun Feb 22 13:54:09 1998
 * Modified at:   Thu Mar  5 14:52:51 1998
 * Modified by:   Dag Brattli <dagb@cs.uit.no>
 * 
 *     Copyright (c) 1998 Dag Brattli, All Rights Reserved.
 *      
 *     This program is free software; you can redistribute it and/or 
 *     modify it under the terms of the GNU General Public License as 
 *     published by the Free Software Foundation; either version 2 of 
 *     the License, or (at your option) any later version.
 *  
 *     Neither Dag Brattli nor University of Troms admit liability nor
 *     provide warranty for any of this software. This material is 
 *     provided "AS-IS" and at no charge.
 *     
 ********************************************************************/

#ifndef IR_COMPAT_H
#define IR_COMPAT_H

#include <linux/version.h>
#include <asm/segment.h>

#define LinuxVersionCode(v, p, s) (((v)<<16)+((p)<<8)+(s))

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,25)
#define test_and_clear_bit clear_bit
#define test_and_set_bit set_bit
#endif

#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,0)
#define copy_from_user(data,buffer,count) memcpy_fromfs(data, buffer, count);
#define copy_to_user(data,buffer,count) memcpy_tofs(data, buffer, count);
#endif

#if LINUX_VERSION_CODE > LinuxVersionCode(2,1,0)
#define proc_register_dynamic proc_register 
#endif

/*
 *  From linux-2.1.86, dev_kfree_skb only needs one parameter
 */
#if LINUX_VERSION_CODE < LinuxVersionCode(2,1,86)
#define DEV_KFREE_SKB(skb,mode) dev_kfree_skb(skb,mode)
#else
#define FREE_READ       1
#define FREE_WRITE      0
#define DEV_KFREE_SKB(skb,mode) dev_kfree_skb(skb)
#endif

#endif /* IR_COMPAT_H */

